import argparse
import json
import os

from PIL import Image, ImageDraw
from tqdm import tqdm


class MeasureVisualizer:
    """
        Class that can be used to visualize the measure annotations that are provided as json files.
        Allows to enable/disable whether to draw system-measures (one measure for all instruments), stave-measures
        (one measure, single stave of one instrument) and staves (the whole line of a single instrument).
        System-measures contain all stave-measures that are being played simulatenously.

        Bounding boxes will be drawn with semitransparent colors.
    """

    def __init__(self, draw_system_measures: bool, draw_stave_measures: bool, draw_staves: bool) -> None:
        super().__init__()
        self.draw_system_measures = draw_system_measures
        self.draw_stave_measures = draw_stave_measures
        self.draw_staves = draw_staves

    def draw_bounding_boxes_for_all_images_in_directory(self, image_directory, json_annotation_directory):
        image_paths = os.listdir(image_directory)
        annotation_paths = os.listdir(json_annotation_directory)

        for image_path in tqdm(image_paths, desc="Drawing annotated images"):
            image_name = os.path.splitext(os.path.basename(image_path))[0]
            annotations = [a for a in annotation_paths if image_name in a]
            # In case of a missing annotation, we skip that image by looping over the result, which would be an empty list
            for annotation_path in annotations:
                self.draw_bounding_boxes_into_image(os.path.join(image_directory, image_path),
                                                    os.path.join(json_annotation_directory, annotation_path))

    def draw_bounding_boxes_into_image(self, image_path: str, ground_truth_annotations_path: str):
        destination_path = image_path.replace(".png", "_annotated.png")
        image = Image.open(image_path)
        image = image.convert("RGB")

        with open(ground_truth_annotations_path, 'r') as gt_file:
            data = json.load(gt_file)

        blue = (0, 0, 255, 100)  # RGBA
        yellow = (255, 255, 0, 100)  # RGBA
        magenta = (255, 0, 255, 100)  # RGBA
        if self.draw_system_measures:
            self._draw_rectangles(data["system_measures"], image, blue)

        if self.draw_stave_measures:
            self._draw_rectangles(data["stave_measures"], image, magenta)

        if self.draw_staves:
            self._draw_rectangles(data["staves"], image, yellow)

        image.save(destination_path)

    def _draw_rectangles(self, rectangles, image, color):
        for rectangle in rectangles:
            left, top, bottom, right = rectangle["left"], rectangle["top"], rectangle["bottom"], \
                                       rectangle["right"]

            # String to float, float to int
            left = int(float(left))
            top = int(float(top))
            bottom = int(float(bottom))
            right = int(float(right))

            image_draw = ImageDraw.Draw(image, "RGBA")
            image_draw.rectangle([left, top, right, bottom], color)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Draw the bounding boxes from the ground-truth data.')
    parser.add_argument('--image_directory', dest='image_directory', type=str, help='Path to the images',
                        default="../data/muscima_pp/v1.0/data/images")
    parser.add_argument('--json_annotation_directory', dest='json_annotation_directory', type=str,
                        default="../data/muscima_pp/v1.0/data/json",
                        help='Path to the JSON annotations that contain the ground truth.')
    parser.add_argument('--system-measures', dest='system_measures', action='store_true')
    parser.add_argument('--no-system-measures', dest='system_measures', action='store_false')
    parser.set_defaults(system_measures=True)
    parser.add_argument('--stave-measures', dest='stave_measures', action='store_true')
    parser.add_argument('--no-stave-measures', dest='stave_measures', action='store_false')
    parser.set_defaults(stave_measures=True)
    parser.add_argument('--staves', dest='staves', action='store_true')
    parser.add_argument('--no-staves', dest='staves', action='store_false')
    parser.set_defaults(staves=True)
    args = parser.parse_args()

    visualizer = MeasureVisualizer(args.system_measures, args.stave_measures, args.staves)
    visualizer.draw_bounding_boxes_for_all_images_in_directory(args.image_directory, args.json_annotation_directory)



import argparse
import os
from enum import Enum
from glob import glob
from typing import List, Tuple

import numpy
from PIL import Image
from mung.io import read_nodes_from_file, parse_node_classes
from mung.node import Node
from tqdm import tqdm


class MaskType(Enum):
    """ The type of masks that should be generated """

    #: creates mask images, where each type of node gets the same color mask (for semantic segmentation).
    #: The classes staffLine, staff and staffSpace are ignored.
    NODES_SEMANTIC_SEGMENTATION = 1

    #: creates mask images, where the masks of the staff lines are contained for instance segmentation.
    #: All five lines that form a staff will have the same color.
    STAFF_LINES_INSTANCE_SEGMENTATION = 2

    #: creates mask images, where each staff will receive one big blob (filling the staff space regions) per staff
    #: line for instance segmentation. So each staff will have a different color.
    STAFF_BLOBS_INSTANCE_SEGMENTATION = 3


class MuscimaPlusPlusMaskImageGenerator:
    def __init__(self) -> None:
        self.path_of_this_file = os.path.dirname(os.path.realpath(__file__))
        self.class_to_color_mapping = dict()

    def render_node_masks(self, raw_data_directory: str, destination_directory: str, mask_type: MaskType):
        """
        Extracts all symbols from the raw XML documents and generates individual symbols from the masks

        :param raw_data_directory: The directory, that contains the xml-files and matching images
        :param destination_directory: The directory, in which the symbols should be generated into.
                                      Per file, one mask will be generated.
        :param mask_type: The type of masks that you want to generate, e.g., masks for each node or staff lines only.
        """
        print("Extracting Masks from Muscima++ Dataset...")

        node_classes = parse_node_classes(
            os.path.join(raw_data_directory, "v2.0", "specifications", "mff-muscima-mlclasses-annot.xml"))
        for index, node_class in enumerate(node_classes):
            self.class_to_color_mapping[node_class.name] = index + 1

        file_paths = self.__get_all_file_paths(raw_data_directory)
        for xml_file, png_file in tqdm(file_paths, desc="Generating mask images"):
            original_image = Image.open(png_file)  # type: Image.Image
            nodes = read_nodes_from_file(xml_file)
            destination_filename = os.path.basename(xml_file).replace(".xml", ".png")
            if mask_type == MaskType.NODES_SEMANTIC_SEGMENTATION:
                self.__render_masks_of_nodes_for_semantic_segmentation(nodes, destination_directory,
                                                                       destination_filename,
                                                                       original_image.width, original_image.height)
            if mask_type == MaskType.STAFF_LINES_INSTANCE_SEGMENTATION:
                self.__render_masks_of_staff_lines_for_instance_segmentation(nodes, destination_directory,
                                                                             destination_filename,
                                                                             original_image.width,
                                                                             original_image.height)
            if mask_type == MaskType.STAFF_BLOBS_INSTANCE_SEGMENTATION:
                self.__render_masks_of_staff_blob_for_instance_segmentation(nodes, destination_directory,
                                                                            destination_filename,
                                                                            original_image.width, original_image.height)
            original_image.close()

    def __get_all_file_paths(self, raw_data_directory: str) -> List[Tuple[str, str]]:
        """ Loads all XML-files that are located in the folder.
        :param raw_data_directory: Path to the raw directory, where the MUSCIMA++ dataset was extracted to
        """
        annotations_directory = os.path.join(raw_data_directory, "v2.0", "data", "annotations")
        xml_files = [y for x in os.walk(annotations_directory) for y in glob(os.path.join(x[0], '*.xml'))]
        images_directory = os.path.join(raw_data_directory, "v2.0", "data", "images")
        png_files = [y for x in os.walk(images_directory) for y in glob(os.path.join(x[0], '*.png'))]
        return list(zip(xml_files, png_files))

    def __render_masks_of_nodes_for_semantic_segmentation(self, nodes: List[Node], destination_directory: str,
                                                          destination_filename: str,
                                                          width: int, height: int):
        image = numpy.zeros((height, width), dtype=numpy.uint8)
        skipped_classes = ["staffSpace", "staff", "staffLine"]
        for node in reversed(nodes):
            if node.class_name in skipped_classes:
                continue
            try:
                symbol_class = node.class_name
                color_mask = node.mask * self.class_to_color_mapping[symbol_class]
                for i in range(node.height):
                    for j in range(node.width):
                        if color_mask[i, j] != 0:
                            image[node.top + i, node.left + j] = color_mask[i, j]
            except:
                print("Error drawing node {0}".format(node.unique_id))

        image = Image.fromarray(image, mode="L")
        os.makedirs(destination_directory, exist_ok=True)
        image.save(os.path.join(destination_directory, destination_filename))

    def __render_masks_of_staff_lines_for_instance_segmentation(self, nodes: List[Node], destination_directory: str,
                                                                destination_filename: str,
                                                                width: int, height: int):
        image = numpy.zeros((height, width), dtype=numpy.uint8)
        included_classes = ["staffLine"]
        staff_line_index = 0
        staff_index = 1
        for node in nodes:
            if node.class_name not in included_classes:
                continue
            if staff_line_index == 5:
                staff_line_index = 0
                staff_index += 1
            staff_line_index += 1
            try:
                color_mask = node.mask * staff_index
                for i in range(node.height):
                    for j in range(node.width):
                        if color_mask[i, j] != 0:
                            image[node.top + i, node.left + j] = color_mask[i, j]
            except:
                print("Error drawing node {0}".format(node.unique_id))

        image = Image.fromarray(image, mode="L")
        os.makedirs(destination_directory, exist_ok=True)
        image.save(os.path.join(destination_directory, destination_filename))

    def __render_masks_of_staff_blob_for_instance_segmentation(self, nodes: List[Node], destination_directory: str,
                                                               destination_filename: str,
                                                               width: int, height: int):
        image = numpy.zeros((height, width), dtype=numpy.uint8)
        included_classes = ["staffLine"]
        staff_line_index = 0
        staff_index = 1
        for node in nodes:
            if node.class_name not in included_classes:
                continue

            if staff_line_index == 4:
                try:
                    for i in range(first_staff_line_of_staff.top, node.bottom):
                        for j in range(node.left, node.right):
                            image[i, j] = staff_index
                except:
                    print("Error drawing node {0}".format(node.unique_id))

            if staff_line_index == 5:
                staff_index += 1
                staff_line_index = 0

            staff_line_index += 1
            if staff_line_index == 1:
                first_staff_line_of_staff = node

        image = Image.fromarray(image, mode="L")
        os.makedirs(destination_directory, exist_ok=True)
        image.save(os.path.join(destination_directory, destination_filename))


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--raw_dataset_directory",
        type=str,
        default="../data/muscima_pp",
        help="The directory, where the raw Muscima++ dataset can be found")
    parser.add_argument(
        "--image_dataset_directory",
        type=str,
        default="../data/muscima_pp_masks",
        help="The directory, where the generated bitmaps will be created")
    parser.add_argument(
        "--mask_type",
        type=str,
        default="nodes_semantic",
        help="One of the following types to be generated: [nodes_semantic, staff_lines, staff_blob]. Depending on the selected type, different"
             "mask images will be created: "
             "- nodes_semantic, creates mask images, where each type of node gets the same color mask (for semantic segmentation). The"
             "classes staffLine, staff and staffSpace are ignored"
             "- staff_lines, creates mask images, where the masks of the staff lines are contained for instance segmentation. All five "
             "lines that form a staff will have the same color."
             "- staff_blob, creates mask images, where each staff will receive one big blob (filling the staff space regions) per staff"
             "line for instance segmentation. So each staff will have a different color.")

    flags, unparsed = parser.parse_known_args()

    if flags.mask_type not in ["nodes_semantic", "staff_lines", "staff_blob"]:
        raise Exception(
            "Invalid option for mask type selected. Must be one of [nodes_semantic, staff_lines, staff_blob], but was " + flags.mask_type)

    if flags.mask_type == "nodes_semantic":
        mask_type = MaskType.NODES_SEMANTIC_SEGMENTATION
    if flags.mask_type == "staff_lines":
        mask_type = MaskType.STAFF_LINES_INSTANCE_SEGMENTATION
    if flags.mask_type == "staff_blob":
        mask_type = MaskType.STAFF_BLOBS_INSTANCE_SEGMENTATION

    mask_image_generator = MuscimaPlusPlusMaskImageGenerator()
    # noinspection PyUnboundLocalVariable
    mask_image_generator.render_node_masks(flags.raw_dataset_directory,
                                           flags.image_dataset_directory,
                                           mask_type)



from enum import Enum, auto
from typing import Dict


class OmrDataset(Enum):
    """
        The available OMR datasets that can be automatically downloaded with Downloader.py
    """

    #: The Audiveris OMR dataset from https://github.com/Audiveris/omr-dataset-tools, Copyright 2017 by Hervé Bitteur under AGPL-3.0 license
    Audiveris = auto()

    #: The Baro Single Stave dataset from http://www.cvc.uab.es/people/abaro/datasets.html, Copyright 2019 Arnau Baró, Pau Riba, Jorge Calvo-Zaragoza, and Alicia Fornés under CC-BY-NC-SA 4.0 license
    Baro = auto()

    #: The Capitan dataset from http://grfia.dlsi.ua.es/, License unspecified, free for research purposes
    Capitan = auto()

    #: Custom version of the CVC-MUSCIMA dataset that contains all images in grayscale, binary and with the
    #: following staff-line augmentations: interrupted, kanungo, thickness-variation-v1/2, y-variation-v1/2
    #: typeset-emulation and whitespeckles. (all data augmentations that could be aligned automatically).
    #: The grayscale images are different from the WriterIdentification dataset, in such a way, that they were aligned
    #: to the images from the Staff-Removal dataset. This is the recommended dataset for object detection, as the
    #: MUSCIMA++ annotations can be used with a variety of underlying images.
    #: See https://github.com/apacha/CVC-MUSCIMA to learn more.
    CvcMuscima_MultiConditionAligned = auto()

    #: The larger version of the CVC-MUSCIMA dataset for staff removal in black and white with augmentations
    #: from http://www.cvc.uab.es/cvcmuscima/index_database.html,
    #: Copyright 2012 Alicia Fornés, Anjan Dutta, Albert Gordo and Josep Lladós under CC-BY-NC-SA 4.0 license
    CvcMuscima_StaffRemoval = auto()

    #: The smaller version of the CVC-MUSCIMA dataset for writer identification in grayscale
    #: from http://www.cvc.uab.es/cvcmuscima/index_database.html,
    #: Copyright 2012 Alicia Fornés, Anjan Dutta, Albert Gordo and Josep Lladós under CC-BY-NC-SA 4.0 license
    CvcMuscima_WriterIdentification = auto()

    #: Edirom dataset. All rights reserved
    Edirom_Bargheer = auto()

    #: Edirom datasets on Freischuetz from https://freischuetz-digital.de/edition.html. All rights reserved.
    Edirom_FreischuetzDigital = auto()

    #: The Fornes Music Symbols dataset from http://www.cvc.uab.es/~afornes/, License unspecified - citation requested
    Fornes = auto()

    #: The official HOMUS dataset from http://grfia.dlsi.ua.es/homus/, License unspecified.
    Homus_V1 = auto()

    #: The improved version of the HOMUS dataset with several bugs-fixed from https://github.com/apacha/Homus
    Homus_V2 = auto()

    #: The MUSCIMA++ dataset from https://ufal.mff.cuni.cz/muscima, Copyright 2017 Jan Hajic jr. under CC-BY-NC-SA 4.0 license
    MuscimaPlusPlus_V1 = auto()

    #: The second version of the MUSCIMA++ dataset from https://github.com/OMR-Research/muscima-pp
    MuscimaPlusPlus_V2 = auto()

    #: The measure annotations from the MUSCIMA++ dataset
    MuscimaPlusPlus_MeasureAnnotations = auto()

    #: The OpenOMR Symbols dataset from https://sourceforge.net/projects/openomr/, Copyright 2013 by Arnaud F. Desaedeleer under GPL license
    OpenOmr = auto()

    #: The Printed Music Symbols dataset from https://github.com/apacha/PrintedMusicSymbolsDataset, Copyright 2017 by Alexander Pacha under MIT license
    Printed = auto()

    #: The Rebelo dataset (part 1) with music symbols from http://www.inescporto.pt/~arebelo/index.php, Copyright 2017 by Ana Rebelo under CC BY-SA 4.0 license
    Rebelo1 = auto()

    #: The Rebelo dataset (part 2) with music symbols from http://www.inescporto.pt/~arebelo/index.php, Copyright 2017 by Ana Rebelo under CC BY-SA 4.0 license
    Rebelo2 = auto()

    def get_dataset_download_url(self) -> str:
        """ Returns the url of the selected dataset.
            Example usage: OmrDataset.Fornes.get_dataset_download_url() """
        return self.dataset_download_urls()[self.name]

    def get_dataset_filename(self) -> str:
        """ Returns the name of the downloaded zip file of a dataset.
            Example usage: OmrDataset.Fornes.get_dataset_filename() """
        return self.dataset_file_names()[self.name]

    def dataset_download_urls(self) -> Dict[str, str]:
        """ Returns a mapping with all URLs, mapped from their enum keys """
        return {
            # Official URL: https://github.com/Audiveris/omr-dataset-tools/tree/master/data/input-images
            "Audiveris": "https://github.com/apacha/OMR-Datasets/releases/download/datasets/AudiverisOmrDataset.zip",

            # Official URL: http://www.cvc.uab.es/people/abaro/datasets/MUSCIMA_ABARO.zip
            "Baro": "https://github.com/apacha/OMR-Datasets/releases/download/datasets/BaroMuscima.zip",

            # Official URL: http://grfia.dlsi.ua.es/cm/projects/timul/databases/BimodalHandwrittenSymbols.zip
            "Capitan": "https://github.com/apacha/OMR-Datasets/releases/download/datasets/BimodalHandwrittenSymbols.zip",

            # Official URL: http://www.cvc.uab.es/cvcmuscima/CVCMUSCIMA_WI.zip
            "CvcMuscima_WriterIdentification": "https://github.com/apacha/OMR-Datasets/releases/download/datasets/CVCMUSCIMA_WI.zip",

            # Official URL: http://www.cvc.uab.es/cvcmuscima/CVCMUSCIMA_SR.zip
            "CvcMuscima_StaffRemoval": "https://github.com/apacha/OMR-Datasets/releases/download/datasets/CVCMUSCIMA_SR.zip",

            # Official URL: https://github.com/apacha/CVC-MUSCIMA
            "CvcMuscima_MultiConditionAligned": "https://github.com/apacha/OMR-Datasets/releases/download/datasets/CVCMUSCIMA_MCA.zip",

            "Edirom_Bargheer": "https://github.com/apacha/OMR-Datasets/releases/download/datasets/Bargheer.zip",

            "Edirom_FreischuetzDigital": "https://github.com/apacha/OMR-Datasets/releases/download/datasets/FreischuetzDigital.zip",

            # Official URL: http://www.cvc.uab.es/cvcmuscima/datasets/Music_Symbols.zip
            "Fornes": "https://github.com/apacha/OMR-Datasets/releases/download/datasets/Music_Symbols.zip",

            # Official URL: http://grfia.dlsi.ua.es/homus/HOMUS.zip
            "Homus_V1": "https://github.com/apacha/OMR-Datasets/releases/download/datasets/HOMUS.zip",

            # Official URL: https://github.com/apacha/Homus
            "Homus_V2": "https://github.com/apacha/OMR-Datasets/releases/download/datasets/HOMUS-2.0.zip",

            # Official URL: https://lindat.mff.cuni.cz/repository/xmlui/bitstream/handle/11372/LRT-2372/MUSCIMA-pp_v1.0.zip?sequence=1&isAllowed=y
            "MuscimaPlusPlus_V1": "https://github.com/OMR-Research/muscima-pp/releases/download/v1.0/MUSCIMA-pp_v1.0.zip",

            # Official URL: https://github.com/OMR-Research/muscima-pp
            "MuscimaPlusPlus_V2": "https://github.com/OMR-Research/muscima-pp/releases/download/v2.0/MUSCIMA-pp_v2.0.zip",

            "MuscimaPlusPlus_Images": "https://github.com/apacha/OMR-Datasets/releases/download/datasets/CVC_MUSCIMA_PP_Annotated-Images.zip",

            "MuscimaPlusPlus_MeasureAnnotations": "https://github.com/apacha/OMR-Datasets/releases/download/datasets/MUSCIMA-pp_v1.0-measure-annotations.zip",

            # Official URL: https://sourceforge.net/projects/openomr/
            "OpenOmr": "https://github.com/apacha/OMR-Datasets/releases/download/datasets/OpenOMR-Dataset.zip",

            "Printed": "https://github.com/apacha/OMR-Datasets/releases/download/datasets/PrintedMusicSymbolsDataset.zip",

            "Rebelo1": "https://github.com/apacha/OMR-Datasets/releases/download/datasets/Rebelo-Music-Symbol-Dataset1.zip",
            "Rebelo2": "https://github.com/apacha/OMR-Datasets/releases/download/datasets/Rebelo-Music-Symbol-Dataset2.zip",
        }

    def dataset_file_names(self) -> Dict[str, str]:
        """ Returns a map of all file_names, mapped from their enum keys """
        return {
            "Audiveris": "AudiverisOmrDataset.zip",
            "Baro": "BaroMuscima.zip",
            "Capitan": "BimodalHandwrittenSymbols.zip",
            "CvcMuscima_WriterIdentification": "CVCMUSCIMA_WI.zip",
            "CvcMuscima_StaffRemoval": "CVCMUSCIMA_SR.zip",
            "CvcMuscima_MultiConditionAligned": "CVCMUSCIMA_MCA.zip",
            "Edirom_Bargheer": "Bargheer.zip",
            "Edirom_FreischuetzDigital": "FreischuetzDigital.zip",
            "Fornes": "Music_Symbols.zip",
            "Homus_V1": "HOMUS.zip",
            "Homus_V2": "HOMUS-2.0.zip",
            "MuscimaPlusPlus_V1": "MUSCIMA-pp_v1.0.zip",
            "MuscimaPlusPlus_V2": "MUSCIMA-pp_v2.0.zip",
            "MuscimaPlusPlus_Images": "CVC_MUSCIMA_PP_Annotated-Images.zip",
            "MuscimaPlusPlus_MeasureAnnotations": "MUSCIMA-pp_v1.0-measure-annotations.zip",
            "OpenOmr": "OpenOMR-Dataset.zip",
            "Printed": "PrintedMusicSymbolsDataset.zip",
            "Rebelo1": "Rebelo-Music-Symbol-Dataset1.zip",
            "Rebelo2": "Rebelo-Music-Symbol-Dataset2.zip",
        }



import os
import unittest

from sympy import Point2D

from omrdatasettools.ExportPath import ExportPath
from omrdatasettools.HomusImageGenerator import HomusSymbol
from omrdatasettools.Rectangle import Rectangle


class HomusSymbolTest(unittest.TestCase):
    def test_initialize_from_empty_string_expect_empty_symbol(self):
        # Arrange

        # Act
        symbol = HomusSymbol.initialize_from_string("")

        # Assert
        self.assertIsNone(symbol)

    def test_initialize_with_simple_symbol(self):
        # Arrange
        content = "test\n23,107;30,101;"
        strokes = [[Point2D(23, 107), Point2D(30, 101)]]
        dimensions = Rectangle(Point2D(23, 101), 8, 7)

        # Act
        symbol = HomusSymbol.initialize_from_string(content)

        # Assert
        self.assertIsNotNone(symbol)
        self.assertEqual("test", symbol.symbol_class)
        self.assertEqual(content, symbol.content)
        self.assertEqual(strokes, symbol.strokes)
        self.assertEqual(dimensions, symbol.dimensions)

    def test_initialize_with_real_symbol(self):
        content = "12-8-Time\n28,107;30,107;30,110;25,121;19,131;13,139;10,144;9,145;10,145;10,145;\n35,115;35," \
                  "115;37,115;39,115;42,116;43,117;42,120;39,124;33,131;30,134;29,137;32,138;36,139;41,140;42,140;42," \
                  "141;42,141;\n26,159;27,159;30,158;36,158;38,159;38,160;35,163;31,166;24,172;21,176;20,180;21," \
                  "182;23,182;26,178;26,175;25,171;24,166;23,159;20,158;20,158;"

        # Act
        symbol = HomusSymbol.initialize_from_string(content)

        # Assert
        self.assertIsNotNone(symbol)
        self.assertEqual("12-8-Time", symbol.symbol_class)
        self.assertEqual(content, symbol.content)
        self.assertEqual(3, len(symbol.strokes), "We expected three strokes")

    def test_draw_onto_canvas(self):
        # Arrange
        symbol = HomusSymbol("", [[Point2D(0, 0), Point2D(100, 100)]], "", Rectangle(Point2D(0, 0), 100, 100))
        export_path = ExportPath("", "", "bitmap", "png", 2)

        # Act
        symbol.draw_onto_canvas(export_path, stroke_thickness=2, margin=2, destination_width=150,
                                destination_height=150)

        # Assert
        self.assertTrue(os.path.exists(export_path.get_full_path()))

        # Cleanup
        os.remove(export_path.get_full_path())

    def test_draw_onto_canvas_random_position(self):
        # Arrange
        symbol = HomusSymbol("", [[Point2D(0, 0), Point2D(100, 100)]], "", Rectangle(Point2D(0, 0), 100, 100))
        export_path = ExportPath("", "", "bitmap_random", "png", 2)
        bounding_boxes = dict()

        # Act
        symbol.draw_onto_canvas(export_path, stroke_thickness=2, margin=2, destination_width=1000,
                                destination_height=1000, random_position_on_canvas=True, bounding_boxes=bounding_boxes)

        # Assert
        self.assertTrue(os.path.exists(export_path.get_full_path()))
        bounding_box = bounding_boxes["bitmap_random_2.png"]
        self.assertEqual(bounding_box.height, 100)
        self.assertEqual(bounding_box.width, 100)
        self.assertNotEqual(bounding_box.left, 450)
        self.assertNotEqual(bounding_box.top, 450)
        self.assertNotEqual(bounding_box.right, 550)
        self.assertNotEqual(bounding_box.bottom, 550)

        # Cleanup
        os.remove(export_path.get_full_path())

    def test_draw_into_bitmap_without_larger_canvas(self):
        # Arrange
        symbol = HomusSymbol("", [[Point2D(0, 0), Point2D(100, 100)]], "", Rectangle(Point2D(0, 0), 100, 100))
        export_path = ExportPath("", "", "bitmap", "png", 3)

        # Act
        symbol.draw_into_bitmap(export_path, stroke_thickness=3, margin=2)

        # Assert
        self.assertTrue(os.path.exists(export_path.get_full_path()))

        # Cleanup
        os.remove(export_path.get_full_path())

    def test_draw_staff_lines(self):
        content = "Quarter-Note\n144,130;144,130;143,130;141,130;139,130;138,131;138,131;138,133;140,135;143,135;146," \
                  "135;150,132;154,128;155,126;153,124;150,124;146,124;142,125;140,127;140,128;141,129;144,130;149," \
                  "129;152,128;153,126;153,124;150,123;147,122;144,123;142,125;140,126;141,128;143,128;146,127;149," \
                  "126;151,124;151,122;148,121;145,122;141,125;138,129;137,131;138,132;141,132;144,131;147,128;148," \
                  "126;148,125;147,125;145,126;144,127;144,129;144,129;147,130;149,130;150,129;150,128;150,127;148," \
                  "127;147,128;147,129;147,130;148,131;151,131;153,130;153,129;154,127;152,126;150,126;148,126;147," \
                  "127;147,128;147,129;148,130;148,130;148,130;146,130;144,130;141,131;140,131;139,131;139,131;139," \
                  "131;139,131;139,131;140,131;140,133;141,137;141,143;141,150;141,158;139,172;139,180;139,188;140," \
                  "192;141,195;142,196;143,196;144,196;144,196;144,197;144,197;"
        export_path = ExportPath("", "", "test", "png", 3)
        symbol = HomusSymbol.initialize_from_string(content)

        # Act
        offsets = [18 + 7 * i for i in range(3)]  # [18,25,32]
        bounding_boxes = dict()
        symbol.draw_onto_canvas(export_path, 3, 0, 128, 224, 14, offsets, bounding_boxes)

        # Assert
        bounding_box_in_image = bounding_boxes["test_3_offset_25.png"]
        self.assertEqual(bounding_box_in_image.origin, Point2D(109 / 2, 147 / 2))
        self.assertEqual(bounding_box_in_image.width, 19)
        self.assertEqual(bounding_box_in_image.height, 77)

        # Cleanup
        for offset in offsets:
            os.remove(export_path.get_full_path(offset))


if __name__ == '__main__':
    unittest.main()



import os


class ExportPath:
    """ An internal helper class to automatically build path names when generating images from annotations with variations """

    def __init__(self, destination_directory: str, symbol_class: str, raw_file_name_without_extension: str,
                 extension: str = "png", stroke_thickness: int = None) -> None:
        super().__init__()
        self.stroke_thickness = stroke_thickness
        self.extension = extension
        self.raw_file_name_without_extension = raw_file_name_without_extension
        self.symbol_class = symbol_class
        self.destination_directory = destination_directory

    def get_full_path(self, offset: int = None):
        """
        :return: Returns the full path that will join all fields according to the following format if no offset if provided:
        'destination_directory'/'symbol_class'/'raw_file_name_without_extension'_'stroke_thickness'.'extension',
        e.g.: data/images/3-4-Time/1-13_3.png

        or with an additional offset-appendix if an offset is provided
        'destination_directory'/'symbol_class'/'raw_file_name_without_extension'_'stroke_thickness'_offset_'offset'.'extension',
        e.g.: data/images/3-4-Time/1-13_3_offset_74.png
        """
        stroke_thickness = ""
        if self.stroke_thickness is not None:
            stroke_thickness = "_{0}".format(self.stroke_thickness)

        staffline_offset = ""
        if offset is not None:
            staffline_offset = "_offset_{0}".format(offset)

        return os.path.join(self.destination_directory, self.symbol_class,
                            "{0}{1}{2}.{3}".format(self.raw_file_name_without_extension,
                                                   stroke_thickness, staffline_offset, self.extension))

    def get_class_name_and_file_path(self, offset: int = None):

        staffline_offset = ""
        if offset is not None:
            staffline_offset = "_offset_{0}".format(offset)

        return os.path.join(self.symbol_class, "{0}_{1}{2}.{3}".format(self.raw_file_name_without_extension,
                                                                       self.stroke_thickness, staffline_offset,
                                                                       self.extension))



import argparse
import os
from glob import glob
from xml.etree import ElementTree

from PIL import Image
from sympy import Point2D

from omrdatasettools.ExportPath import ExportPath
from omrdatasettools.Rectangle import Rectangle


class AudiverisOmrSymbol(Rectangle):
    def __init__(self, symbol_class: str, x: int, y: int, width: int, height: int) -> None:
        super().__init__(Point2D(x, y), width, height)
        self.symbol_class = symbol_class


class AudiverisOmrImageGenerator:
    def __init__(self) -> None:
        super().__init__()

    def extract_symbols(self, raw_data_directory: str, destination_directory: str):
        """
        Extracts the symbols from the raw XML documents and matching images of the Audiveris OMR dataset into
        individual symbols

        :param raw_data_directory: The directory, that contains the xml-files and matching images
        :param destination_directory: The directory, in which the symbols should be generated into. One sub-folder per
                                      symbol category will be generated automatically
        """
        print("Extracting Symbols from Audiveris OMR Dataset...")

        all_xml_files = [y for x in os.walk(raw_data_directory) for y in glob(os.path.join(x[0], '*.xml'))]
        all_image_files = [y for x in os.walk(raw_data_directory) for y in glob(os.path.join(x[0], '*.png'))]

        data_pairs = []
        for i in range(len(all_xml_files)):
            data_pairs.append((all_xml_files[i], all_image_files[i]))

        for data_pair in data_pairs:
            self.__extract_symbols(data_pair[0], data_pair[1], destination_directory)

    def __extract_symbols(self, xml_file: str, image_file: str, destination_directory: str):
        # xml_file, image_file = 'data/audiveris_omr_raw\\IMSLP06053p1.xml', 'data/audiveris_omr_raw\\IMSLP06053p1.png'
        # xml_file, image_file = 'data/audiveris_omr_raw\\mops-1.xml', 'data/audiveris_omr_raw\\mops-1.png'
        # xml_file, image_file = 'data/audiveris_omr_raw\\mtest1-1.xml', 'data/audiveris_omr_raw\\mtest1-1.png'
        # xml_file, image_file = 'data/audiveris_omr_raw\\mtest2-1.xml', 'data/audiveris_omr_raw\\mtest2-1.png'
        image = Image.open(image_file)
        annotations = ElementTree.parse(xml_file).getroot()
        xml_symbols = annotations.findall("Symbol")

        file_name_without_extension = os.path.splitext(os.path.basename(xml_file))[0]
        symbols = []

        for xml_symbol in xml_symbols:
            symbol_class = xml_symbol.get("shape")

            bounds = xml_symbol.find("Bounds")
            x, y, width, height = bounds.get("x"), bounds.get("y"), bounds.get("w"), bounds.get("h")
            x, y, width, height = int(float(x)), int(float(y)), int(float(width)), int(float(height))

            symbol = AudiverisOmrSymbol(symbol_class, x, y, width, height)
            symbols.append(symbol)

        symbol_number = 0
        for symbol in symbols:
            symbol_class = symbol.symbol_class
            bounding_box_with_one_pixel_margin = symbol.as_bounding_box_with_margin(1)
            symbol_image = image.crop(bounding_box_with_one_pixel_margin)

            target_directory = os.path.join(destination_directory, symbol_class)
            os.makedirs(target_directory, exist_ok=True)

            export_path = ExportPath(destination_directory, symbol_class,
                                     file_name_without_extension + str(symbol_number))
            symbol_image.save(export_path.get_full_path())
            symbol_number += 1


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--raw_dataset_directory",
        type=str,
        default="../data/audiveris_omr_raw",
        help="The directory, where the raw Audiveris OMR dataset can be found")
    parser.add_argument(
        "--image_dataset_directory",
        type=str,
        default="../data/audiveris_omr",
        help="The directory, where the generated bitmaps will be created")

    flags, unparsed = parser.parse_known_args()

    audiveris_omr_image_generator = AudiverisOmrImageGenerator()
    audiveris_omr_image_generator.extract_symbols(flags.raw_dataset_directory, flags.image_dataset_directory)



import os
import shutil
import unittest
from glob import glob

from omrdatasettools.Downloader import Downloader
from omrdatasettools.OmrDataset import OmrDataset


class DownloaderTest(unittest.TestCase):

    def test_download_correct_url_resolution(self):
        downloader = OmrDataset.Audiveris
        url = downloader.get_dataset_download_url()
        expected_url = "https://github.com/apacha/OMR-Datasets/releases/download/datasets/AudiverisOmrDataset.zip"

        self.assertEqual(url, expected_url)

    def test_download_of_audiveris_dataset(self):
        destination_directory = "AudiverisRawData"
        dataset = OmrDataset.Audiveris
        number_of_samples_in_the_dataset = 4
        target_file_extension = "*.png"

        self.download_dataset_and_verify_correct_extraction(destination_directory, number_of_samples_in_the_dataset,
                                                            target_file_extension, dataset)

    def test_download_of_capitan_dataset(self):
        destination_directory = "CapitanRawData"
        dataset = OmrDataset.Capitan
        number_of_samples_in_the_dataset = 1
        target_file_extension = "data"

        self.download_dataset_and_verify_correct_extraction(destination_directory, number_of_samples_in_the_dataset,
                                                            target_file_extension, dataset)

    def test_download_of_baro_dataset(self):
        destination_directory = "Baro"
        dataset = OmrDataset.Baro
        number_of_samples_in_the_dataset = 212
        target_file_extension = "*.txt"

        self.download_dataset_and_verify_correct_extraction(destination_directory, number_of_samples_in_the_dataset,
                                                            target_file_extension, dataset)

    @unittest.skip("Only run, if you have time to kill")
    def test_download_of_cvc_muscima_writer_identification_dataset(self):
        destination_directory = "CvcMuscimaWriterIdentificationData"
        dataset = OmrDataset.CvcMuscima_WriterIdentification
        number_of_samples_in_the_dataset = 3000
        target_file_extension = "*.png"

        self.download_dataset_and_verify_correct_extraction(destination_directory, number_of_samples_in_the_dataset,
                                                            target_file_extension, dataset)

    @unittest.skip("Only run, if you have time to kill")
    def test_download_of_cvc_muscima_staff_removal_dataset(self):
        destination_directory = "CvcMuscimaStaffRemovalData"
        dataset = OmrDataset.CvcMuscima_StaffRemoval
        number_of_samples_in_the_dataset = 36000
        target_file_extension = "*.png"

        self.download_dataset_and_verify_correct_extraction(destination_directory, number_of_samples_in_the_dataset,
                                                            target_file_extension, dataset)

    @unittest.skip("Only run, if you have time to kill")
    def test_download_of_cvc_muscima_multi_conditional_aligned_dataset(self):
        destination_directory = "CvcMuscimaMultiConditionAlignedData"
        dataset = OmrDataset.CvcMuscima_MultiConditionAligned
        number_of_samples_in_the_dataset = 10000
        target_file_extension = "*.png"

        self.download_dataset_and_verify_correct_extraction(destination_directory, number_of_samples_in_the_dataset,
                                                            target_file_extension, dataset)

    def test_download_of_fornes_dataset(self):
        destination_directory = "FornesMusicSymbols"
        dataset = OmrDataset.Fornes
        number_of_samples_in_the_dataset = 4094
        target_file_extension = "*.bmp"

        self.download_dataset_and_verify_correct_extraction(destination_directory, number_of_samples_in_the_dataset,
                                                            target_file_extension, dataset)

    def test_download_and_extract_bargheer_edirom_dataset_expect_folder_to_be_created(self):
        destination_directory = "Bargheer"
        dataset = OmrDataset.Edirom_Bargheer
        number_of_samples_in_the_dataset = 9
        target_file_extension = "*.xml"

        self.download_dataset_and_verify_correct_extraction(destination_directory, number_of_samples_in_the_dataset,
                                                            target_file_extension, dataset)

    def test_download_and_extract_freischuetz_edirom_dataset_expect_folder_to_be_created(self):
        destination_directory = "FreischuetzDigital"
        dataset = OmrDataset.Edirom_FreischuetzDigital
        number_of_samples_in_the_dataset = 15
        target_file_extension = "*.xml"

        self.download_dataset_and_verify_correct_extraction(destination_directory, number_of_samples_in_the_dataset,
                                                            target_file_extension, dataset)

    def test_download_of_homus_v1_dataset(self):
        destination_directory = "HOMUS"
        dataset = OmrDataset.Homus_V1
        number_of_samples_in_the_dataset = 15200
        target_file_extension = "*.txt"

        self.download_dataset_and_verify_correct_extraction(destination_directory, number_of_samples_in_the_dataset,
                                                            target_file_extension, dataset)

    def test_download_of_homus_v2_dataset(self):
        destination_directory = "HOMUS2"
        dataset = OmrDataset.Homus_V2
        number_of_samples_in_the_dataset = 15200
        target_file_extension = "*.txt"

        self.download_dataset_and_verify_correct_extraction(destination_directory, number_of_samples_in_the_dataset,
                                                            target_file_extension, dataset)

    def test_download_of_muscima_pp_v1_dataset(self):
        destination_directory = "MuscimaPlusPlus"
        dataset = OmrDataset.MuscimaPlusPlus_V1
        number_of_samples_with_staff_lines = 140
        number_of_samples_without_staff_lines = 140
        extra_files = 1
        number_of_xml_files_in_the_dataset = number_of_samples_with_staff_lines + number_of_samples_without_staff_lines + extra_files
        target_file_extension = "*.xml"

        self.download_dataset_and_verify_correct_extraction(destination_directory, number_of_xml_files_in_the_dataset,
                                                            target_file_extension, dataset)

    def test_download_of_muscima_pp_v2_dataset(self):
        destination_directory = "MuscimaPlusPlus"
        dataset = OmrDataset.MuscimaPlusPlus_V2
        number_of_samples_with_staff_lines = 140
        extra_files = 1
        number_of_xml_files_in_the_dataset = number_of_samples_with_staff_lines + extra_files
        target_file_extension = "*.xml"

        self.download_dataset_and_verify_correct_extraction(destination_directory, number_of_xml_files_in_the_dataset,
                                                            target_file_extension, dataset)

    def test_download_of_muscima_pp_v1_images(self):
        destination_directory = "MuscimaPlusPlus"
        dataset = OmrDataset.MuscimaPlusPlus_V1
        number_of_images = 140
        target_file_extension = "*.png"

        self.download_dataset_and_verify_correct_extraction(destination_directory, number_of_images,
                                                            target_file_extension, dataset)

    def test_download_of_muscima_pp_v2_images(self):
        destination_directory = "MuscimaPlusPlus"
        dataset = OmrDataset.MuscimaPlusPlus_V2
        number_of_images = 140
        target_file_extension = "*.png"

        self.download_dataset_and_verify_correct_extraction(destination_directory, number_of_images,
                                                            target_file_extension, dataset)

    def test_download_of_muscima_pp_measure_annotations(self):
        destination_directory = "MuscimaPlusPlus"
        dataset = OmrDataset.MuscimaPlusPlus_MeasureAnnotations
        number_of_annotation_files = 144
        target_file_extension = "*.json"

        self.download_dataset_and_verify_correct_extraction(destination_directory, number_of_annotation_files,
                                                            target_file_extension, dataset)

    def test_download_of_openomr_dataset(self):
        destination_directory = "OpenOMR"
        dataset = OmrDataset.OpenOmr
        number_of_samples_in_the_dataset = 706
        target_file_extension = "*.png"

        self.download_dataset_and_verify_correct_extraction(destination_directory, number_of_samples_in_the_dataset,
                                                            target_file_extension, dataset)

    def test_download_of_printed_symbols_dataset(self):
        destination_directory = "PrintedMusicSymbols"
        dataset = OmrDataset.Printed
        number_of_samples_in_the_dataset = 213
        target_file_extension = "*.png"

        self.download_dataset_and_verify_correct_extraction(destination_directory, number_of_samples_in_the_dataset,
                                                            target_file_extension, dataset)

    def test_download_of_rebelo1_dataset(self):
        destination_directory = "Rebelo1Images"
        dataset = OmrDataset.Rebelo1
        number_of_samples_in_the_dataset = 7940
        target_file_extension = "*.png"

        self.download_dataset_and_verify_correct_extraction(destination_directory, number_of_samples_in_the_dataset,
                                                            target_file_extension, dataset)

    def test_download_of_rebelo2_dataset(self):
        destination_directory = "Rebelo2Images"
        dataset = OmrDataset.Rebelo2
        number_of_samples_in_the_dataset = 7307
        target_file_extension = "*.png"

        self.download_dataset_and_verify_correct_extraction(destination_directory, number_of_samples_in_the_dataset,
                                                            target_file_extension, dataset)

    def download_dataset_and_verify_correct_extraction(self: unittest.TestCase, destination_directory: str,
                                                       number_of_samples_in_the_dataset: int,
                                                       target_file_extension: str, dataset: OmrDataset):
        # Arrange and Cleanup
        if os.path.exists(dataset.get_dataset_filename()):
            os.remove(dataset.get_dataset_filename())
        shutil.rmtree(destination_directory, ignore_errors=True)
        downloader = Downloader()

        # Act
        downloader.download_and_extract_dataset(dataset, destination_directory)

        # Assert
        all_files = glob(destination_directory + "/**/" + target_file_extension, recursive=True)
        actual_number_of_files = len(all_files)
        self.assertEqual(number_of_samples_in_the_dataset, actual_number_of_files)
        self.assertTrue(os.path.exists(dataset.get_dataset_filename()))

        # Cleanup
        os.remove(dataset.get_dataset_filename())
        shutil.rmtree(destination_directory, ignore_errors=True)

        if __name__ == '__main__':
            unittest.main()



import os
import shutil
import unittest
from glob import glob

from omrdatasettools.Downloader import Downloader
from omrdatasettools.HomusImageGenerator import HomusImageGenerator
from omrdatasettools.OmrDataset import OmrDataset


class HomusImageGeneratorTest(unittest.TestCase):
    def test_download_extract_and_draw_bitmaps(self):
        # Arrange
        datasetDownloader = Downloader()

        # Act
        datasetDownloader.download_and_extract_dataset(OmrDataset.Homus_V2, "temp/homus_raw")
        HomusImageGenerator.create_images("temp/homus_raw", "temp/homus_img", [3], 96, 192, 14)
        all_image_files = [y for x in os.walk("temp/homus_img") for y in glob(os.path.join(x[0], '*.png'))]
        actual_number_of_files = len(all_image_files)

        # Assert
        self.assertEqual(15200, actual_number_of_files)

        # Cleanup
        os.remove("HOMUS-2.0.zip")
        shutil.rmtree("temp")


if __name__ == '__main__':
    unittest.main()



import argparse
import os
from glob import glob
from typing import List

from PIL import Image
from mung.io import read_nodes_from_file
from mung.node import Node
from tqdm import tqdm

from omrdatasettools.ExportPath import ExportPath


class MuscimaPlusPlusSymbolImageGenerator:
    def __init__(self) -> None:
        super().__init__()
        self.path_of_this_file = os.path.dirname(os.path.realpath(__file__))

    def extract_and_render_all_symbol_masks(self, raw_data_directory: str, destination_directory: str):
        """
        Extracts all symbols from the raw XML documents and generates individual symbols from the masks

        :param raw_data_directory: The directory, that contains the xml-files and matching images
        :param destination_directory: The directory, in which the symbols should be generated into. One sub-folder per
                                      symbol category will be generated automatically
        """
        print("Extracting Symbols from MUSCIMA++ Dataset...")

        xml_files = self.get_all_xml_file_paths(raw_data_directory)
        crop_objects = self.load_nodes_from_xml_files(xml_files)
        self.render_masks_of_nodes_into_image(crop_objects, destination_directory)

    def get_all_xml_file_paths(self, raw_data_directory: str) -> List[str]:
        """ Loads all XML-files that are located in the folder.
        :param raw_data_directory: Path to the raw directory, where the MUSCIMA++ dataset was extracted to
        """
        raw_data_directory = os.path.join(raw_data_directory, "v2.0", "data", "annotations")
        xml_files = [y for x in os.walk(raw_data_directory) for y in glob(os.path.join(x[0], '*.xml'))]
        return xml_files

    def load_nodes_from_xml_files(self, xml_files: List[str]) -> List[Node]:
        nodes = []  # type: List[Node]
        for xml_file in tqdm(xml_files, desc="Loading nodes from xml-files", smoothing=0.1):
            nodes.extend(read_nodes_from_file(xml_file))

        print("Loaded {0} nodes".format(len(nodes)))
        return nodes

    def render_masks_of_nodes_into_image(self, nodes: List[Node], destination_directory: str):
        for node in tqdm(nodes, desc="Generating images from node masks", smoothing=0.1):  # type: Node
            symbol_class = node.class_name
            # Make a copy of the mask to not temper with the original data
            mask = node.mask.copy()
            # We want to draw black symbols on white canvas. The mask encodes foreground pixels
            # that we are interested in with a 1 and background pixels with a 0 and stores those values in
            # an uint8 numpy array. To use Image.fromarray, we have to generate a greyscale mask, where
            # white pixels have the value 255 and black pixels have the value 0. To achieve this, we simply
            # subtract one from each uint, and by exploiting the underflow of the uint we get the following mapping:
            # 0 (background) => 255 (white) and 1 (foreground) => 0 (black) which is exactly what we wanted.
            mask -= 1
            image = Image.fromarray(mask, mode="L")

            target_directory = os.path.join(destination_directory, symbol_class)
            os.makedirs(target_directory, exist_ok=True)

            export_path = ExportPath(destination_directory, symbol_class, node.unique_id)
            image.save(export_path.get_full_path())


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--raw_dataset_directory",
        type=str,
        default="../data/muscima_pp",
        help="The directory, where the Muscima++ dataset can be found. Must be at least version 2.0.")
    parser.add_argument(
        "--image_dataset_directory",
        type=str,
        default="../data/muscima_pp/symbols",
        help="The directory, where the generated bitmaps will be created")

    flags, unparsed = parser.parse_known_args()

    muscima_pp_image_generator = MuscimaPlusPlusSymbolImageGenerator()
    muscima_pp_image_generator.extract_and_render_all_symbol_masks(flags.raw_dataset_directory,
                                                                   flags.image_dataset_directory)



import argparse
import os
from typing import List

import numpy
from PIL import Image, ImageDraw
from sympy import Point2D
from tqdm import tqdm

from omrdatasettools.ExportPath import ExportPath
from omrdatasettools.Rectangle import Rectangle


class SimplePoint2D(object):
    def __init__(self, x: float, y: float) -> None:
        self.x = x
        self.y = y


class CapitanSymbol:
    def __init__(self, content: str, stroke: List[SimplePoint2D], image_data: numpy.ndarray, symbol_class: str,
                 dimensions: Rectangle) -> None:
        super().__init__()
        self.dimensions = dimensions
        self.symbol_class = symbol_class
        self.content = content
        self.stroke = stroke
        self.image_data = image_data

    @staticmethod
    def initialize_from_string(content: str) -> 'CapitanSymbol':
        """
        Create and initializes a new symbol from a string
        :param content: The content of a symbol as read from the text-file in the form <label>:<sequence>:<image>
        :return: The initialized symbol
        :rtype: CapitanSymbol
        """

        if content is None or content is "":
            return None

        parts = content.split(":")
        min_x = 100000
        max_x = 0
        min_y = 100000
        max_y = 0

        symbol_name = parts[0]

        sequence = parts[1]
        image_numbers = parts[2].split(',')
        image_data = numpy.asarray(image_numbers, numpy.uint8).reshape((30, 30))

        stroke = []

        for point_string in sequence.split(";"):
            if point_string is "":
                continue  # Skip the last element, that is due to a trailing ; in each line

            point_x, point_y = point_string.split(",")
            x = float(point_x)
            y = float(point_y)
            stroke.append(SimplePoint2D(x, y))

            max_x = max(max_x, x)
            min_x = min(min_x, x)
            max_y = max(max_y, y)
            min_y = min(min_y, y)

        dimensions = Rectangle(Point2D(min_x, min_y), int(max_x - min_x + 1), int(max_y - min_y + 1))
        return CapitanSymbol(content, stroke, image_data, symbol_name, dimensions)

    def draw_capitan_score_bitmap(self, export_path: ExportPath) -> None:
        """
        Draws the 30x30 symbol into the given file
        :param export_path: The path, where the symbols should be created on disk
        """
        with Image.fromarray(self.image_data, mode='L') as image:
            image.save(export_path.get_full_path())

    def draw_capitan_stroke_onto_canvas(self, export_path: ExportPath, stroke_thickness: int, margin: int):
        """
        Draws the symbol strokes onto a canvas
        :param export_path: The path, where the symbols should be created on disk
        :param stroke_thickness:
        :param margin:
        """
        width = int(self.dimensions.width + 2 * margin)
        height = int(self.dimensions.height + 2 * margin)
        offset = Point2D(self.dimensions.origin.x - margin, self.dimensions.origin.y - margin)

        image = Image.new('RGB', (width, height), "white")  # create a new white image
        draw = ImageDraw.Draw(image)
        black = (0, 0, 0)

        for i in range(0, len(self.stroke) - 1):
            start_point = self.__subtract_offset(self.stroke[i], offset)
            end_point = self.__subtract_offset(self.stroke[i + 1], offset)
            distance = self.__euclidean_distance(start_point, end_point)
            if distance > 1600:  # User moved more than 40 pixels - probably we should not draw a line here
                continue
            draw.line((start_point.x, start_point.y, end_point.x, end_point.y), black, stroke_thickness)

        del draw

        image.save(export_path.get_full_path())
        image.close()

    @staticmethod
    def __euclidean_distance(a: SimplePoint2D, b: SimplePoint2D) -> float:
        return (a.x - b.x) * (a.x - b.x) + abs(a.y - b.y) * abs(a.y - b.y)

    @staticmethod
    def __manhatten_distance(a: SimplePoint2D, b: SimplePoint2D) -> float:
        return abs(a.x - b.x) + abs(a.y - b.y)

    @staticmethod
    def __subtract_offset(a: SimplePoint2D, b: SimplePoint2D) -> SimplePoint2D:
        return SimplePoint2D(a.x - b.x, a.y - b.y)


class CapitanImageGenerator:
    def create_capitan_images(self, raw_data_directory: str,
                              destination_directory: str,
                              stroke_thicknesses: List[int]) -> None:
        """
        Creates a visual representation of the Capitan strokes by parsing all text-files and the symbols as specified
        by the parameters by drawing lines that connect the points from each stroke of each symbol.

        :param raw_data_directory: The directory, that contains the raw capitan dataset
        :param destination_directory: The directory, in which the symbols should be generated into. One sub-folder per
                                      symbol category will be generated automatically
        :param stroke_thicknesses: The thickness of the pen, used for drawing the lines in pixels. If multiple are
                                   specified, multiple images will be generated that have a different suffix, e.g.
                                   1-16-3.png for the 3-px version and 1-16-2.png for the 2-px version of the image 1-16
        """
        symbols = self.load_capitan_symbols(raw_data_directory)
        self.draw_capitan_stroke_images(symbols, destination_directory, stroke_thicknesses)
        self.draw_capitan_score_images(symbols, destination_directory)

    def load_capitan_symbols(self, raw_data_directory: str) -> List[CapitanSymbol]:
        data_path = os.path.join(raw_data_directory, "BimodalHandwrittenSymbols", "data")
        with open(data_path) as file:
            data = file.read()

        symbol_strings = data.splitlines()
        symbols = []
        for symbol_string in tqdm(symbol_strings, desc="Loading symbols from strings"):
            symbol = CapitanSymbol.initialize_from_string(symbol_string)
            symbols.append(symbol)

        return symbols

    def draw_capitan_stroke_images(self, symbols: List[CapitanSymbol],
                                   destination_directory: str,
                                   stroke_thicknesses: List[int]) -> None:
        """
        Creates a visual representation of the Capitan strokes by drawing lines that connect the points
        from each stroke of each symbol.

        :param symbols: The list of parsed Capitan-symbols
        :param destination_directory: The directory, in which the symbols should be generated into. One sub-folder per
                                      symbol category will be generated automatically
        :param stroke_thicknesses: The thickness of the pen, used for drawing the lines in pixels. If multiple are
                                   specified, multiple images will be generated that have a different suffix, e.g.
                                   1-16-3.png for the 3-px version and 1-16-2.png for the 2-px version of the image 1-16
        """

        total_number_of_symbols = len(symbols) * len(stroke_thicknesses)
        output = "Generating {0} images with {1} symbols in {2} different stroke thicknesses ({3})".format(
            total_number_of_symbols, len(symbols), len(stroke_thicknesses), stroke_thicknesses)

        print(output)
        print("In directory {0}".format(os.path.abspath(destination_directory)), flush=True)

        progress_bar = tqdm(total=total_number_of_symbols, mininterval=0.25, desc="Rendering strokes")
        capitan_file_name_counter = 0
        for symbol in symbols:
            capitan_file_name_counter += 1
            target_directory = os.path.join(destination_directory, symbol.symbol_class)
            os.makedirs(target_directory, exist_ok=True)

            raw_file_name_without_extension = "capitan-{0}-{1}-stroke".format(symbol.symbol_class,
                                                                              capitan_file_name_counter)

            for stroke_thickness in stroke_thicknesses:
                export_path = ExportPath(destination_directory, symbol.symbol_class, raw_file_name_without_extension,
                                         'png', stroke_thickness)
                symbol.draw_capitan_stroke_onto_canvas(export_path, stroke_thickness, 0)
                progress_bar.update(1)

        progress_bar.close()

    def draw_capitan_score_images(self, symbols: List[CapitanSymbol],
                                  destination_directory: str) -> None:
        """
        Draws the image data contained in each symbol

        :param symbols: The list of parsed Capitan-symbols
        :param destination_directory: The directory, in which the symbols should be generated into. One sub-folder per
                                      symbol category will be generated automatically
        :param stroke_thicknesses: The thickness of the pen, used for drawing the lines in pixels. If multiple are
                                   specified, multiple images will be generated that have a different suffix, e.g.
                                   1-16-3.png for the 3-px version and 1-16-2.png for the 2-px version of the image 1-16
        """

        total_number_of_symbols = len(symbols)
        output = "Generating {0} images from Capitan symbols".format(len(symbols))

        print(output)
        print("In directory {0}".format(os.path.abspath(destination_directory)), flush=True)

        progress_bar = tqdm(total=total_number_of_symbols, mininterval=0.25, desc="Rendering images")
        capitan_file_name_counter = 0
        for symbol in symbols:
            capitan_file_name_counter += 1
            target_directory = os.path.join(destination_directory, symbol.symbol_class)
            os.makedirs(target_directory, exist_ok=True)

            raw_file_name_without_extension = "capitan-{0}-{1}-score".format(symbol.symbol_class,
                                                                             capitan_file_name_counter)

            export_path = ExportPath(destination_directory, symbol.symbol_class, raw_file_name_without_extension, 'png')
            symbol.draw_capitan_score_bitmap(export_path)
            progress_bar.update(1)

        progress_bar.close()

    @staticmethod
    def add_arguments_for_homus_image_generator(parser: argparse.ArgumentParser):
        parser.add_argument("-s", "--stroke_thicknesses", dest="stroke_thicknesses", default="3",
                            help="Stroke thicknesses for drawing the generated bitmaps. May define comma-separated list"
                                 " of multiple stroke thicknesses, e.g. '1,2,3'")


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--raw_dataset_directory",
        type=str,
        default="../data/capitan_raw",
        help="The directory, where the raw HOMUS dataset can be found (the text-files that contain the strokes)")
    parser.add_argument(
        "--image_dataset_directory",
        type=str,
        default="../data/images",
        help="The directory, where the generated bitmaps will be created")

    image_generator = CapitanImageGenerator()
    image_generator.add_arguments_for_homus_image_generator(parser)

    flags, unparsed = parser.parse_known_args()

    image_generator.create_capitan_images(flags.raw_dataset_directory, flags.image_dataset_directory,
                                          [int(s) for s in flags.stroke_thicknesses.split(',')])



from typing import Tuple

from sympy import Point2D


class Rectangle:
    def __init__(self, origin: Point2D, width: int, height: int):
        super().__init__()
        self.height = height
        self.width = width
        self.origin = origin  # Resembles the left top point
        self.left = origin.x
        self.top = origin.y
        self.right = self.left + self.width
        self.bottom = self.top + self.height

    @staticmethod
    def overlap(r1: 'Rectangle', r2: 'Rectangle'):
        """
        Overlapping rectangles overlap both horizontally & vertically
        """
        h_overlaps = (r1.left <= r2.right) and (r1.right >= r2.left)
        v_overlaps = (r1.bottom >= r2.top) and (r1.top <= r2.bottom)
        return h_overlaps and v_overlaps

    @staticmethod
    def merge(r1: 'Rectangle', r2: 'Rectangle') -> 'Rectangle':
        left = min(r1.left, r2.left)
        top = min(r1.top, r2.top)
        right = max(r1.right, r2.right)
        bottom = max(r1.bottom, r2.bottom)
        width = right - left
        height = bottom - top

        return Rectangle(Point2D(left, top), width, height)

    def as_bounding_box_with_margin(self, margin: int = 1) -> Tuple[int, int, int, int]:
        bounding_box_with_margin = (self.left - margin,
                                    self.top - margin,
                                    self.left + self.width + 2 * margin,
                                    self.top + self.height + 2 * margin)
        return bounding_box_with_margin

    def __eq__(self, o: object) -> bool:
        are_equal = self.width == o.width and self.height == o.height and self.origin == o.origin
        return are_equal

    def __str__(self) -> str:
        return "Rectangle[Origin:{0},{1}, Width:{2}, Height:{3}]".format(self.origin.x, self.origin.y, self.width,
                                                                         self.height)



import argparse
import os
import random
import sys
from glob import glob
from typing import List

from PIL import Image, ImageDraw
from sympy import Point2D
from tqdm import tqdm

from omrdatasettools.ExportPath import ExportPath
from omrdatasettools.Rectangle import Rectangle


class HomusSymbol:
    def __init__(self, content: str, strokes: List[List[Point2D]], symbol_class: str, dimensions: Rectangle) -> None:
        super().__init__()
        self.dimensions = dimensions
        self.symbol_class = symbol_class
        self.content = content
        self.strokes = strokes

    @staticmethod
    def initialize_from_string(content: str) -> 'HomusSymbol':
        """
        Create and initializes a new symbol from a string

        :param content: The content of a symbol as read from the text-file
        :return: The initialized symbol
        :rtype: HomusSymbol
        """

        if content is None or content is "":
            return None

        lines = content.splitlines()
        min_x = sys.maxsize
        max_x = 0
        min_y = sys.maxsize
        max_y = 0

        symbol_name = lines[0]
        strokes = []

        for stroke_string in lines[1:]:
            stroke = []

            for point_string in stroke_string.split(";"):
                if point_string is "":
                    continue  # Skip the last element, that is due to a trailing ; in each line

                point_x, point_y = point_string.split(",")
                x = int(point_x)
                y = int(point_y)
                stroke.append(Point2D(x, y))

                max_x = max(max_x, x)
                min_x = min(min_x, x)
                max_y = max(max_y, y)
                min_y = min(min_y, y)

            strokes.append(stroke)

        dimensions = Rectangle(Point2D(min_x, min_y), max_x - min_x + 1, max_y - min_y + 1)
        return HomusSymbol(content, strokes, symbol_name, dimensions)

    def draw_into_bitmap(self, export_path: ExportPath, stroke_thickness: int, margin: int = 0) -> None:
        """
        Draws the symbol in the original size that it has plus an optional margin

        :param export_path: The path, where the symbols should be created on disk
        :param stroke_thickness: Pen-thickness for drawing the symbol in pixels
        :param margin: An optional margin for each symbol
        """
        self.draw_onto_canvas(export_path,
                              stroke_thickness,
                              margin,
                              self.dimensions.width + 2 * margin,
                              self.dimensions.height + 2 * margin)

    def draw_onto_canvas(self, export_path: ExportPath, stroke_thickness: int, margin: int, destination_width: int,
                         destination_height: int, staff_line_spacing: int = 14,
                         staff_line_vertical_offsets: List[int] = None,
                         bounding_boxes: dict = None, random_position_on_canvas: bool = False) -> None:
        """
        Draws the symbol onto a canvas with a fixed size

        :param bounding_boxes: The dictionary into which the bounding-boxes will be added of each generated image
        :param export_path: The path, where the symbols should be created on disk
        :param stroke_thickness:
        :param margin:
        :param destination_width:
        :param destination_height:
        :param staff_line_spacing:
        :param staff_line_vertical_offsets: Offsets used for drawing staff-lines. If None provided, no staff-lines will be drawn if multiple integers are provided, multiple images will be generated
        """
        width = self.dimensions.width + 2 * margin
        height = self.dimensions.height + 2 * margin
        if random_position_on_canvas:
            # max is required for elements that are larger than the canvas,
            # where the possible range for the random value would be negative
            random_horizontal_offset = random.randint(0, max(0, destination_width - width))
            random_vertical_offset = random.randint(0, max(0, destination_height - height))
            offset = Point2D(self.dimensions.origin.x - margin - random_horizontal_offset,
                             self.dimensions.origin.y - margin - random_vertical_offset)
        else:
            width_offset_for_centering = (destination_width - width) / 2
            height_offset_for_centering = (destination_height - height) / 2
            offset = Point2D(self.dimensions.origin.x - margin - width_offset_for_centering,
                             self.dimensions.origin.y - margin - height_offset_for_centering)

        image_without_staff_lines = Image.new('RGB', (destination_width, destination_height),
                                              "white")  # create a new white image
        draw = ImageDraw.Draw(image_without_staff_lines)
        black = (0, 0, 0)

        for stroke in self.strokes:
            for i in range(0, len(stroke) - 1):
                start_point = self.__subtract_offset(stroke[i], offset)
                end_point = self.__subtract_offset(stroke[i + 1], offset)
                draw.line((start_point.x, start_point.y, end_point.x, end_point.y), black, stroke_thickness)

        location = self.__subtract_offset(self.dimensions.origin, offset)
        bounding_box_in_image = Rectangle(location, self.dimensions.width, self.dimensions.height)
        # self.draw_bounding_box(draw, location)

        del draw

        if staff_line_vertical_offsets is not None and staff_line_vertical_offsets:
            for staff_line_vertical_offset in staff_line_vertical_offsets:
                image_with_staff_lines = image_without_staff_lines.copy()
                self.__draw_staff_lines_into_image(image_with_staff_lines, stroke_thickness,
                                                   staff_line_spacing, staff_line_vertical_offset)
                file_name_with_offset = export_path.get_full_path(staff_line_vertical_offset)
                image_with_staff_lines.save(file_name_with_offset)
                image_with_staff_lines.close()

                if bounding_boxes is not None:
                    # Note that the ImageDatasetGenerator does not yield the full path, but only the class_name and
                    # the file_name, e.g. '3-4-Time\\1-13_3_offset_74.png', so we store only that part in the dictionary
                    class_and_file_name = export_path.get_class_name_and_file_path(staff_line_vertical_offset)
                    bounding_boxes[class_and_file_name] = bounding_box_in_image
        else:
            image_without_staff_lines.save(export_path.get_full_path())
            if bounding_boxes is not None:
                # Note that the ImageDatasetGenerator does not yield the full path, but only the class_name and
                # the file_name, e.g. '3-4-Time\\1-13_3_offset_74.png', so we store only that part in the dictionary
                class_and_file_name = export_path.get_class_name_and_file_path()
                bounding_boxes[class_and_file_name] = bounding_box_in_image

        image_without_staff_lines.close()

    def draw_bounding_box(self, draw, location):
        red = (255, 0, 0)
        draw.rectangle(
            (location.x, location.y, location.x + self.dimensions.width, location.y + self.dimensions.height),
            fill=None, outline=red)

    @staticmethod
    def __draw_staff_lines_into_image(image: Image,
                                      stroke_thickness: int,
                                      staff_line_spacing: int = 14,
                                      vertical_offset=88):
        black = (0, 0, 0)
        width = image.width
        draw = ImageDraw.Draw(image)

        for i in range(5):
            y = vertical_offset + i * staff_line_spacing
            draw.line((0, y, width, y), black, stroke_thickness)
        del draw

    @staticmethod
    def __subtract_offset(a: Point2D, b: Point2D) -> Point2D:
        return Point2D(a.x - b.x, a.y - b.y)


class HomusImageGenerator:
    @staticmethod
    def create_images(raw_data_directory: str,
                      destination_directory: str,
                      stroke_thicknesses: List[int],
                      canvas_width: int = None,
                      canvas_height: int = None,
                      staff_line_spacing: int = 14,
                      staff_line_vertical_offsets: List[int] = None,
                      random_position_on_canvas: bool = False) -> dict:
        """
        Creates a visual representation of the Homus Dataset by parsing all text-files and the symbols as specified
        by the parameters by drawing lines that connect the points from each stroke of each symbol.

        Each symbol will be drawn in the center of a fixed canvas, specified by width and height.

        :param raw_data_directory: The directory, that contains the text-files that contain the textual representation
                                    of the music symbols
        :param destination_directory: The directory, in which the symbols should be generated into. One sub-folder per
                                      symbol category will be generated automatically
        :param stroke_thicknesses: The thickness of the pen, used for drawing the lines in pixels. If multiple are
                                   specified, multiple images will be generated that have a different suffix, e.g.
                                   1-16-3.png for the 3-px version and 1-16-2.png for the 2-px version of the image 1-16
        :param canvas_width: The width of the canvas, that each image will be drawn upon, regardless of the original size of
                      the symbol. Larger symbols will be cropped. If the original size of the symbol should be used,
                      provided None here.
        :param canvas_height: The height of the canvas, that each image will be drawn upon, regardless of the original size of
                       the symbol. Larger symbols will be cropped. If the original size of the symbol should be used,
                       provided None here
        :param staff_line_spacing: Number of pixels spacing between each of the five staff-lines
        :param staff_line_vertical_offsets: List of vertical offsets, where the staff-lines will be superimposed over
                                            the drawn images. If None is provided, no staff-lines will be superimposed.
                                            If multiple values are provided, multiple versions of each symbol will be
                                            generated with the appropriate staff-lines, e.g. 1-5_3_offset_70.png and
                                            1-5_3_offset_77.png for two versions of the symbol 1-5 with stroke thickness
                                            3 and staff-line offsets 70 and 77 pixels from the top.
        :param random_position_on_canvas: True, if the symbols should be randomly placed on the fixed canvas.
                                          False, if the symbols should be centered in the fixed canvas.
                                          Note that this flag only has an effect, if fixed canvas sizes are used.
        :return: A dictionary that contains the file-names of all generated symbols and the respective bounding-boxes
                 of each symbol.
        """
        all_symbol_files = [y for x in os.walk(raw_data_directory) for y in glob(os.path.join(x[0], '*.txt'))]

        staff_line_multiplier = 1
        if staff_line_vertical_offsets is not None and staff_line_vertical_offsets:
            staff_line_multiplier = len(staff_line_vertical_offsets)

        total_number_of_symbols = len(all_symbol_files) * len(stroke_thicknesses) * staff_line_multiplier
        output = "Generating {0} images with {1} symbols in {2} different stroke thicknesses ({3})".format(
            total_number_of_symbols, len(all_symbol_files), len(stroke_thicknesses), stroke_thicknesses)

        if staff_line_vertical_offsets is not None:
            output += " and with staff-lines with {0} different offsets from the top ({1})".format(
                staff_line_multiplier, staff_line_vertical_offsets)

        if canvas_width is not None and canvas_height is not None:
            if random_position_on_canvas is False:
                output += "\nRandomly drawn on a fixed canvas of size {0}x{1} (Width x Height)".format(canvas_width,
                                                                                                       canvas_height)
            else:
                output += "\nCentrally drawn on a fixed canvas of size {0}x{1} (Width x Height)".format(canvas_width,
                                                                                                        canvas_height)

        print(output)
        print("In directory {0}".format(os.path.abspath(destination_directory)), flush=True)

        bounding_boxes = dict()

        progress_bar = tqdm(total=total_number_of_symbols, mininterval=0.25)
        for symbol_file in all_symbol_files:
            with open(symbol_file) as file:
                content = file.read()

            symbol = HomusSymbol.initialize_from_string(content)

            target_directory = os.path.join(destination_directory, symbol.symbol_class)
            os.makedirs(target_directory, exist_ok=True)

            raw_file_name_without_extension = os.path.splitext(os.path.basename(symbol_file))[0]

            for stroke_thickness in stroke_thicknesses:
                export_path = ExportPath(destination_directory, symbol.symbol_class, raw_file_name_without_extension,
                                         'png', stroke_thickness)
                if canvas_width is None and canvas_height is None:
                    symbol.draw_into_bitmap(export_path, stroke_thickness, margin=2)
                else:
                    symbol.draw_onto_canvas(export_path, stroke_thickness, 0, canvas_width,
                                            canvas_height, staff_line_spacing, staff_line_vertical_offsets,
                                            bounding_boxes, random_position_on_canvas)

                progress_bar.update(1 * staff_line_multiplier)

        progress_bar.close()
        return bounding_boxes

    @staticmethod
    def add_arguments_for_homus_image_generator(parser: argparse.ArgumentParser):
        parser.add_argument("-s", "--stroke_thicknesses", dest="stroke_thicknesses", default="3",
                            help="Stroke thicknesses for drawing the generated bitmaps. May define comma-separated list"
                                 " of multiple stroke thicknesses, e.g. '1,2,3'")
        parser.add_argument("--staff_line_spacing", default="14", type=int,
                            help="Spacing between two staff-lines in pixel")
        parser.add_argument("-offsets", "--staff_line_vertical_offsets", dest="offsets", default="",
                            help="Optional vertical offsets in pixel for drawing the symbols with superimposed "
                                 "staff-lines starting at this pixel-offset from the top. Multiple offsets possible, "
                                 "e.g. '81,88,95'")
        parser.add_argument("--disable_fixed_canvas_size", dest="use_fixed_canvas",
                            action="store_false",
                            help="True, if the images should be drawn on a fixed canvas with the specified width and height."
                                 "False to draw the symbols with their original sizes (each symbol might be different)")
        parser.set_defaults(use_fixed_canvas=True)
        parser.add_argument("--random_position_on_canvas", dest="random_position_on_canvas", action="store_true",
                            help="Provide this flag, if the symbols should be randomly placed on the fixed canvas."
                                 "Omit this flag, if the symbols should be centered in the fixed canvas (default)."
                                 "Note, that this flag only has an effect, if a fixed canvas size is used which gets "
                                 "disabled by the --disable_fixed_canvas_size flag.")
        parser.set_defaults(random_position_on_canvas=False)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--raw_dataset_directory",
        type=str,
        default="../data/homus_raw",
        help="The directory, where the raw HOMUS dataset can be found (the text-files that contain the strokes)")
    parser.add_argument(
        "--image_dataset_directory",
        type=str,
        default="../data/images",
        help="The directory, where the generated bitmaps will be created")

    HomusImageGenerator.add_arguments_for_homus_image_generator(parser)
    parser.add_argument("--width", default="96", type=int, help="Width of the generated images in pixel")
    parser.add_argument("--height", default="96", type=int, help="Height of the generated images in pixel")

    flags, unparsed = parser.parse_known_args()

    offsets = []
    if flags.offsets != "":
        offsets = [int(o) for o in flags.offsets.split(',')]

    width, height = flags.width, flags.height
    if not flags.use_fixed_canvas:
        width, height = None, None

    HomusImageGenerator.create_images(flags.raw_dataset_directory,
                                      flags.image_dataset_directory,
                                      [int(s) for s in flags.stroke_thicknesses.split(',')],
                                      width,
                                      height,
                                      flags.staff_line_spacing,
                                      offsets,
                                      flags.random_position_on_canvas)



from ._version import __version__ as version
from .AudiverisOmrImageGenerator import AudiverisOmrImageGenerator
from .CapitanImageGenerator import CapitanImageGenerator
from .Downloader import *
from .HomusImageGenerator import HomusImageGenerator
from .MeasureVisualizer import MeasureVisualizer
from .MuscimaPlusPlusMaskImageGenerator import MuscimaPlusPlusMaskImageGenerator
from .MuscimaPlusPlusSymbolImageGenerator import MuscimaPlusPlusSymbolImageGenerator
from .OmrDataset import OmrDataset

__version__ = version
__all__ = ['Downloader', 'OmrDataset', 'AudiverisOmrImageGenerator', 'CapitanImageGenerator', 'HomusImageGenerator',
           'MeasureVisualizer', 'MuscimaPlusPlusSymbolImageGenerator', 'MuscimaPlusPlusMaskImageGenerator']



import os
import shutil
import unittest
from glob import glob

from omrdatasettools.Downloader import Downloader
from omrdatasettools.MuscimaPlusPlusSymbolImageGenerator import MuscimaPlusPlusSymbolImageGenerator
from omrdatasettools.OmrDataset import OmrDataset


class MuscimaPlusPlusSymbolImageGeneratorTest(unittest.TestCase):
    def test_download_extract_and_render_all_symbols(self):
        # Arrange
        datasetDownloader = Downloader()

        # Act
        datasetDownloader.download_and_extract_dataset(OmrDataset.MuscimaPlusPlus_V2, "temp/muscima_pp")
        image_generator = MuscimaPlusPlusSymbolImageGenerator()
        image_generator.extract_and_render_all_symbol_masks("temp/muscima_pp", "temp/muscima_pp/symbols")
        all_image_files = [y for x in os.walk("temp/muscima_pp/symbols") for y in glob(os.path.join(x[0], '*.png'))]
        expected_number_of_symbols = 102914
        actual_number_of_symbols = len(all_image_files)

        # Assert
        self.assertEqual(expected_number_of_symbols, actual_number_of_symbols)

        # Cleanup
        os.remove(OmrDataset.MuscimaPlusPlus_V2.get_dataset_filename())
        shutil.rmtree("temp")


if __name__ == '__main__':
    unittest.main()



import os
import shutil
import urllib
import urllib.parse as urlparse
import urllib.request as urllib2
from glob import glob
from zipfile import ZipFile

from lxml import etree
from tqdm import tqdm

from omrdatasettools.OmrDataset import OmrDataset


class Downloader:
    """ The class for downloading OMR datasets. It downloads the selected dataset from Github and extracts it to
        a specified directory.
    """

    def download_and_extract_dataset(self, dataset: OmrDataset, destination_directory: str):
        """ Starts the download of the dataset and extracts it into the specified directory.

        Examples
        --------
        >>> from omrdatasettools import Downloader, OmrDataset
        >>> downloader = Downloader()
        >>> downloader.download_and_extract_dataset(OmrDataset.HOMUS_V2, "data")

        """
        if not os.path.exists(dataset.get_dataset_filename()):
            print("Downloading {0} dataset...".format(dataset.name))
            self.__download_file(dataset.get_dataset_download_url(), dataset.get_dataset_filename())

        print("Extracting {0} dataset...".format(dataset.name))
        self.__extract_dataset(os.path.abspath(destination_directory), dataset.get_dataset_filename())

        if dataset is OmrDataset.Fornes:
            self.__fix_capital_file_endings(os.path.join(os.path.abspath(destination_directory), "Music_Symbols"))

        if dataset in [OmrDataset.MuscimaPlusPlus_V1, OmrDataset.MuscimaPlusPlus_V2]:
            self.__download_muscima_pp_images(dataset, destination_directory)

    def download_images_from_mei_annotation(self, dataset: OmrDataset, dataset_directory: str, base_url: str):
        """ Crawls the images of an Edirom dataset, if provided with the respective URL. To avoid repetitive crawling,
            this URL has to be provided manually. If you are interested in these datasets, please contact the authors.

            Examples
            --------
            >>> from omrdatasettools import Downloader, OmrDataset
            >>> downloader = Downloader()
            >>> downloader.download_and_extract_dataset(OmrDataset.Edirom_Bargheer, "data/Bargheer")
            >>> downloader.download_images_from_mei_annotation(OmrDataset.Edirom_Bargheer, "data/Bargheer", "INSERT_DATASET_URL_HERE")

            or

            >>> downloader.download_and_extract_dataset(OmrDataset.Edirom_FreischuetzDigital, "data/Freischuetz")
            >>> downloader.download_images_from_mei_annotation(OmrDataset.Edirom_FreischuetzDigital, "data/Freischuetz", "INSERT_DATASET_URL_HERE")
            """
        if dataset not in [OmrDataset.Edirom_Bargheer, OmrDataset.Edirom_FreischuetzDigital]:
            raise Exception("Only supported for edirom datasets")

        if len(glob(f'{dataset_directory}/*.xml')) == 0:
            print(
                f"Could not find MEI (XML) files in {dataset_directory}/ directory. Can't download images.")

        for source in glob(f'{dataset_directory}/*.xml'):
            base = os.path.splitext(source)[0]
            os.makedirs(base, exist_ok=True)
            print("Downloading dataset for " + base)
            self.__download_edirom_images(base, base_url, source)

    def __download_edirom_images(self, base, base_url, source):
        xml = etree.parse(source).getroot()

        for graphic in tqdm(xml.xpath('//*[local-name()="graphic"]'), desc="Downloading images"):
            url = graphic.get('target')
            filename = os.path.basename(url)
            width = graphic.get('width')
            if os.path.exists(os.path.join(base, filename)):
                pass  # Skipping download, because it has been downloaded already
            else:
                urllib.request.urlretrieve(f"{base_url}/{url}?dw={width}&amp;mo=fit", os.path.join(base, filename))

    def __download_muscima_pp_images(self, dataset: OmrDataset, destination_directory: str):
        # Automatically download the images and measure annotations with the MUSCIMA++ dataset
        muscima_pp_images_filename = dataset.dataset_file_names()["MuscimaPlusPlus_Images"]
        self.__download_file(dataset.dataset_download_urls()["MuscimaPlusPlus_Images"], muscima_pp_images_filename)
        absolute_path_to_temp_folder = os.path.abspath('MuscimaPpImages')
        self.__extract_dataset(absolute_path_to_temp_folder, muscima_pp_images_filename)
        if dataset is OmrDataset.MuscimaPlusPlus_V1:
            target_folder = os.path.join(os.path.abspath(destination_directory), "v1.0", "data", "images")
        if dataset is OmrDataset.MuscimaPlusPlus_V2:
            target_folder = os.path.join(os.path.abspath(destination_directory), "v2.0", "data", "images")
        self.__copytree(os.path.join(absolute_path_to_temp_folder, "fulls"), target_folder)
        self.__clean_up_temp_directory(absolute_path_to_temp_folder)

    def __fix_capital_file_endings(self, absolute_path_to_temp_folder):
        image_with_capital_file_ending = [y for x in os.walk(absolute_path_to_temp_folder) for y in
                                          glob(os.path.join(x[0], "*.BMP"))]
        for image in image_with_capital_file_ending:
            os.rename(image, image[:-3] + "bmp")

    @staticmethod
    def __copytree(src, dst):
        if not os.path.exists(dst):
            os.makedirs(dst)
        for item in os.listdir(src):
            s = os.path.join(src, item)
            d = os.path.join(dst, item)
            if os.path.isdir(s):
                Downloader.__copytree(s, d)
            else:
                if not os.path.exists(d) or os.stat(s).st_mtime - os.stat(d).st_mtime > 1:
                    shutil.copy2(s, d)

    @staticmethod
    def __extract_dataset(absolute_path_to_folder: str, dataset_filename: str):
        archive = ZipFile(dataset_filename, "r")
        archive.extractall(absolute_path_to_folder)
        archive.close()

    @staticmethod
    def __clean_up_temp_directory(temp_directory):
        print("Deleting temporary directory {0}".format(temp_directory))
        shutil.rmtree(temp_directory, ignore_errors=True)

    @staticmethod
    def __download_file(url, destination_filename=None) -> str:
        u = urllib2.urlopen(url)
        scheme, netloc, path, query, fragment = urlparse.urlsplit(url)
        filename = os.path.basename(path)
        if not filename:
            filename = 'downloaded.file'
        if destination_filename:
            filename = destination_filename

        filename = os.path.abspath(filename)

        with open(filename, 'wb') as f:
            meta = u.info()
            meta_func = meta.getheaders if hasattr(meta, 'getheaders') else meta.get_all
            meta_length = meta_func("Content-Length")
            file_size = None
            if meta_length:
                file_size = int(meta_length[0])
            print("Downloading: {0} Bytes: {1} into {2}".format(url, file_size, filename))

            with tqdm(total=file_size, desc="Downloading (bytes)") as progress_bar:
                file_size_dl = 0
                block_sz = 8192
                while True:
                    buffer = u.read(block_sz)
                    if not buffer:
                        break

                    file_size_dl += len(buffer)
                    f.write(buffer)
                    if file_size:
                        progress_bar.update(len(buffer))
            print()

        return filename



import io
import logging
import os
from os.path import abspath, dirname, join

from setuptools import find_packages
from setuptools import setup

here = os.path.abspath(os.path.dirname(__file__))

# Read the version from
version_path = join(abspath(dirname(__file__)), 'omrdatasettools', '_version.py')
version_dict = {}
exec(open(version_path, 'r').read(), version_dict)
version_number = version_dict['__version__']


def read(*filenames, **kwargs):
    encoding = kwargs.get('encoding', 'utf-8')
    sep = kwargs.get('sep', '\n')
    buf = []
    for filename in filenames:
        with io.open(filename, encoding=encoding) as f:
            buf.append(f.read())
    return sep.join(buf)


def get_long_description():
    readme = os.path.join(here, 'README_omrdatasettools.md')
    changes = os.path.join(here, 'CHANGES.md')

    if os.path.isfile(readme) and os.path.isfile(changes):
        long_description = read(readme, changes)
    else:
        logging.warning('Could not find README.md and CHANGES.md file'
                        ' in directory {0}. Contents:'
                        ' {1}'.format(here, os.listdir(here)))
        long_description = 'A collection of tools that simplify the downloading and handling of datasets used for ' \
                           'Optical Music Recognition (OMR).' \
                           'These tools are available as Python package ``omrdatasettools`` on PyPi.' \
                           '' \
                           'They simplify the most common tasks such as downloading and extracting a dataset, ' \
                           'generating images from textual representations or visualizing those datasets. '
    return long_description


setup(
    name='omrdatasettools',
    packages=find_packages('.'),
    version=version_number,
    description='A collection of tools that simplify the downloading and handling of datasets used for Optical Music '
                'Recognition (OMR).',
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    author='Alexander Pacha',
    author_email='alexander.pacha@tuwien.ac.at',
    license='MIT',
    url='https://github.com/apacha/omr-datasets',  # use the URL to the github repo
    download_url='https://github.com/apacha/OMR-Datasets/archive/{0}.tar.gz'.format(
        version_number),
    keywords=['optical music recognition', 'downloading', 'extracting', 'omr', 'generating',
              'dataset', 'preprocessing'],
    install_requires=['Pillow', 'muscima', "mung", "numpy", "tqdm", "scikit-image", "lxml"],
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',

        'Topic :: Scientific/Engineering :: Image Recognition',
        'Topic :: Scientific/Engineering :: Visualization',
        'Topic :: Multimedia :: Graphics',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)



import os
import shutil
import unittest
from glob import glob

from omrdatasettools.AudiverisOmrImageGenerator import AudiverisOmrImageGenerator
from omrdatasettools.Downloader import Downloader
from omrdatasettools.OmrDataset import OmrDataset


class AudiverisOmrImageGeneratorTest(unittest.TestCase):

    def test_download_extract_and_crop_bitmaps(self):
        # Arrange
        dataset_downloader = Downloader()

        # Act
        dataset_downloader.download_and_extract_dataset(OmrDataset.Audiveris, "temp/audiveris_omr_raw")
        image_generator = AudiverisOmrImageGenerator()
        image_generator.extract_symbols("temp/audiveris_omr_raw", "temp/audiveris_omr_img")
        all_image_files = [y for x in os.walk("temp/audiveris_omr_img") for y in glob(os.path.join(x[0], '*.png'))]
        actual_number_of_files = len(all_image_files)

        # Assert
        self.assertEqual(1056, actual_number_of_files)

        # Cleanup
        os.remove("AudiverisOmrDataset.zip")
        shutil.rmtree("temp")


if __name__ == '__main__':
    unittest.main()



import unittest

from omrdatasettools.ExportPath import ExportPath


class ExportPathTest(unittest.TestCase):
    def test_get_full_path_without_stroke_thickness(self):
        # Arrange
        export_path = ExportPath("data/images", "3-4-Time", "1-13", "png")

        # Act
        full_path = export_path.get_full_path()

        # Assert
        full_path = full_path.replace('\\', '/')
        self.assertEqual("data/images/3-4-Time/1-13.png", full_path)

    def test_get_full_path(self):
        # Arrange
        export_path = ExportPath("data/images", "3-4-Time", "1-13", "png", 3)

        # Act
        full_path = export_path.get_full_path()

        # Assert
        full_path = full_path.replace('\\', '/')
        self.assertEqual("data/images/3-4-Time/1-13_3.png", full_path)

    def test_get_full_path_with_offset(self):
        # Arrange
        export_path = ExportPath("data/images", "3-4-Time", "1-13", "png", 3)

        # Act
        full_path = export_path.get_full_path(33)

        # Assert
        full_path = full_path.replace('\\', '/')
        self.assertEqual("data/images/3-4-Time/1-13_3_offset_33.png", full_path)

    def test_get_class_name_and_file_path(self):
        # Arrange
        export_path = ExportPath("data/images", "3-4-Time", "1-13", "png", 3)

        # Act
        full_path = export_path.get_class_name_and_file_path()

        # Assert
        full_path = full_path.replace('\\', '/')
        self.assertEqual("3-4-Time/1-13_3.png", full_path)

    def test_get_class_name_and_file_path_with_offset(self):
        # Arrange
        export_path = ExportPath("data/images", "3-4-Time", "1-13", "png", 3)

        # Act
        full_path = export_path.get_class_name_and_file_path(33)

        # Assert
        full_path = full_path.replace('\\', '/')
        self.assertEqual("3-4-Time/1-13_3_offset_33.png", full_path)


if __name__ == '__main__':
    unittest.main()



import os
import shutil
import unittest
from glob import glob

from omrdatasettools.MuscimaPlusPlusMaskImageGenerator import \
    MuscimaPlusPlusMaskImageGenerator, MaskType

dir_path = os.path.dirname(os.path.realpath(__file__))


class MuscimaPlusPlusMaskImageGeneratorTest(unittest.TestCase):
    def test_render_node_masks_semantic_segmentation_of_nodes(self):
        # Arrange
        image_generator = MuscimaPlusPlusMaskImageGenerator()

        # Act
        image_generator.render_node_masks(os.path.join(dir_path, "testdata/muscima-pp_v2"),
                                          os.path.join(dir_path, "temp/muscima-pp_v2_masks"),
                                          MaskType.NODES_SEMANTIC_SEGMENTATION)

        # Assert
        all_image_files = [y for x in os.walk(os.path.join(dir_path, "temp/muscima-pp_v2_masks")) for y in
                           glob(os.path.join(x[0], '*.png'))]
        expected_number_of_images = 1
        actual_number_of_images = len(all_image_files)
        self.assertEqual(expected_number_of_images, actual_number_of_images)

        # Cleanup
        shutil.rmtree(os.path.join(dir_path, "temp"))

    def test_render_node_masks_instance_segmentation_of_staff_lines(self):
        # Arrange
        image_generator = MuscimaPlusPlusMaskImageGenerator()

        # Act
        image_generator.render_node_masks(os.path.join(dir_path, "testdata/muscima-pp_v2"),
                                          os.path.join(dir_path, "temp/muscima-pp_v2_masks"),
                                          MaskType.STAFF_LINES_INSTANCE_SEGMENTATION)

        # Assert
        all_image_files = [y for x in os.walk(os.path.join(dir_path, "temp/muscima-pp_v2_masks")) for y in
                           glob(os.path.join(x[0], '*.png'))]
        expected_number_of_images = 1
        actual_number_of_images = len(all_image_files)
        self.assertEqual(expected_number_of_images, actual_number_of_images)

        # Cleanup
        shutil.rmtree(os.path.join(dir_path, "temp"))

    def test_render_node_masks_instance_segmentation_of_staff_blobs(self):
        # Arrange
        image_generator = MuscimaPlusPlusMaskImageGenerator()

        # Act
        image_generator.render_node_masks(os.path.join(dir_path, "testdata/muscima-pp_v2"),
                                          os.path.join(dir_path, "temp/muscima-pp_v2_masks"),
                                          MaskType.STAFF_BLOBS_INSTANCE_SEGMENTATION)

        # Assert
        all_image_files = [y for x in os.walk(os.path.join(dir_path, "temp/muscima-pp_v2_masks")) for y in
                           glob(os.path.join(x[0], '*.png'))]
        expected_number_of_images = 1
        actual_number_of_images = len(all_image_files)
        self.assertEqual(expected_number_of_images, actual_number_of_images)

        # Cleanup
        shutil.rmtree(os.path.join(dir_path, "temp"))


if __name__ == '__main__':
    unittest.main()
